@@ -9,12 +9,12 @@ It can read audio data from audio files as well as from built-in device(s) or st
 
 @author:     Mohamed El Amine SEHILI
 
-@copyright:  2015-2016 Mohamed El Amine SEHILI
+@copyright:  2015-2018 Mohamed El Amine SEHILI
 
 @license:    GPL v3
 
 @contact:    amine.sehili@gmail.com
-@deffield    updated: 13 May 2016
+@deffield    updated: 01 Nov 2018
 '''
 
 import sys
@@ -52,7 +52,7 @@ from auditok import __version__ as version
 __all__ = []
 __version__ = version
 __date__ = '2015-11-23'
-__updated__ = '2016-05-13'
+__updated__ = '2018-10-06'
 
 DEBUG = 0
 TESTRUN = 1
@@ -542,7 +542,7 @@ def main(argv=None):
     program_version_string = '%%prog %s (%s)' % (program_version, program_build_date)
     #program_usage = '''usage: spam two eggs''' # optional - will be autogenerated by optparse
     program_longdesc = '''''' # optional - give further explanation about what the program does
-    program_license = "Copyright 2015 Mohamed El Amine SEHILI                                            \
+    program_license = "Copyright 2015-2018 Mohamed El Amine SEHILI                                            \
                 Licensed under the General Public License (GPL) Version 3 \nhttp://www.gnu.org/licenses/"
 
     if argv is None:
@@ -576,6 +576,8 @@ def main(argv=None):
         group.add_option("-r", "--rate", dest="sampling_rate", help="Sampling rate of audio data [default: %default]", type=int, default=16000, metavar="INT")
         group.add_option("-c", "--channels", dest="channels", help="Number of channels of audio data [default: %default]", type=int, default=1, metavar="INT")
         group.add_option("-w", "--width", dest="sample_width", help="Number of bytes per audio sample [default: %default]", type=int, default=2, metavar="INT")
+        group.add_option("-I", "--input-device-index", dest="input_device_index", help="Audio device index [default: %default] - only when using PyAudio", type=int, default=None, metavar="INT")
+        group.add_option("-F", "--audio-frame-per-buffer", dest="frame_per_buffer", help="Audio frame per buffer [default: %default] - only when using PyAudio", type=int, default=1024, metavar="INT")
         parser.add_option_group(group)
         
         group = OptionGroup(parser, "[Do something with detections]", "Use these options to print, play or plot detections.") 
@@ -609,7 +611,9 @@ def main(argv=None):
             try:
                 asource = PyAudioSource(sampling_rate = opts.sampling_rate,
                                         sample_width = opts.sample_width,
-                                        channels = opts.channels)
+                                        channels = opts.channels,
+                                        frames_per_buffer = opts.frame_per_buffer,
+                                        input_device_index = opts.input_device_index)
             except Exception:
                 sys.stderr.write("Cannot read data from audio device!\n")
                 sys.stderr.write("You should either install pyaudio or read data from STDIN\n")
